
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <assert.h>
#include <bcmnvram.h>
#include <bcmgpio.h>
#include <shutils.h>
#include "Wsc_gpio.h"
#include "tutrace.h"
#include <asm/termbits.h>
#define TCGETS		0x540d
#define TCSETS		0x540e
#define TCSETSW		0x540f
#define TCSETSF		0x5410

#define setTermSettings(fd,argp) tcsetattr(fd,TCSANOW,argp)
#define getTermSettings(fd,argp) tcgetattr(fd, argp);

bcm_timer_module_id wsc_tmodule, wsc_tmodule1;

static int wsc_btn_gpio = BCMGPIO_UNDEFINED;
//static int wsc_led_gpio = BCMGPIO_UNDEFINED;
static int wsc_btn_assertlvl = WSC_BTN_ASSERTLVL;
//static int sc_led_assertlvl = WSC_LED_ASSERTLVL;
static int monitor_console = 0;


int wsc_gpio_btn_init ()
{
	char *value;

	/* Determine the GPIO pins for the WSC Button */

	wsc_btn_gpio = bcmgpio_getpin(WSC_GPIO_BUTTON_VALUE, BCMGPIO_UNDEFINED);

	if (wsc_btn_gpio != BCMGPIO_UNDEFINED) {
		TUTRACE((TUTRACE_INFO,"Using pin %d for wsc button input\n", wsc_btn_gpio));

		if (bcmgpio_connect (wsc_btn_gpio, BCMGPIO_DIRN_IN) != 0) {
			TUTRACE((TUTRACE_INFO,"Error connecting GPIO %d to wsc button\n", 
				wsc_btn_gpio));
			return -1;
		}
		value = nvram_safe_get(WSC_NV_BTN_ASSERTLVL);
		if (strcmp(value, "")) {
			wsc_btn_assertlvl = atoi(value) ? 1 : 0;
			TUTRACE((TUTRACE_INFO,"Using assertlvl %d for wsc button\n", 
				wsc_btn_assertlvl));
		}
	}

	else
	{
		TUTRACE((TUTRACE_INFO,"Not connecting => GPIO %d to wsc button\n", 
				wsc_btn_gpio));
	    return -1;
	}
	/* init timer modules one-time*/
	bcm_timer_module_init(16, &wsc_tmodule);
	bcm_timer_module_init(16, &wsc_tmodule1);

	return 0;
}

void wsc_gpio_btn_cleanup ()
{
	if (wsc_btn_gpio != BCMGPIO_UNDEFINED) {
		bcmgpio_disconnect (wsc_btn_gpio);
		wsc_btn_gpio = BCMGPIO_UNDEFINED;
	}
}

int
brcm_wsc_event_get()
{
	struct timeval time;
	int event = WSC_EVTI_NULL;


	return 0;
	while (1) {
		event = brcm_wsc_retrieve_event();
		if ((event != WSC_EVTI_NULL)) {
			TUTRACE((TUTRACE_INFO,"returning event %d\n", event));
			return(event);
		}

		time.tv_sec = WSC_BUTTON_POLL_TIME / 1000;
		time.tv_usec = (WSC_BUTTON_POLL_TIME % 1000) * 1000;

		/* enable timer handling */
		bcm_timer_module_enable(wsc_tmodule, 1);

		select(0, NULL, NULL, NULL, &time);

		/* disable timer handling */
		bcm_timer_module_enable(wsc_tmodule, 0);
	}
	
}

int
brcm_wsc_retrieve_event()
{
	char *value;
	char buf[4];
	int event = WSC_EVTI_NULL;

	switch (wsc_btn_pressed()) {
		case WSC_NO_BTNPRESS:
			value = nvram_safe_get(WSC_NV_EVENT);
			if ((event = atoi(value)) != WSC_EVTI_NULL) {
				sprintf(buf, "%d", WSC_EVTI_NULL);
				nvram_set(WSC_NV_EVENT, buf);
			}
			break;
		case WSC_SHORT_BTNPRESS:
			TUTRACE((TUTRACE_INFO,"Detected a short wsc button-push\n"));
			event = WSC_EVTI_NW_GEN_WSC;
			break;
		case WSC_LONG_BTNPRESS:
			/* Directly using printf so get indication in external builds */
			TUTRACE((TUTRACE_INFO,"Detected a long wsc button-push\n"));
			event = WSC_EVTI_INIT_CONDITION;
			break;
		default:
			assert(0);
	}

	return event;
}


wsc_btnpress_t wsc_btn_pressed ()
{
	unsigned long btn_mask;
	unsigned long value;
	struct timeval time;
	bool long_btnpress = FALSE;
	static bool first_time = TRUE;
	struct timespec start_ts, end_ts;

	if (wsc_btn_gpio == BCMGPIO_UNDEFINED)
		return WSC_NO_BTNPRESS;

	btn_mask = ((unsigned long)1 << wsc_btn_gpio);

	bcmgpio_in (btn_mask, &value);
	value >>= wsc_btn_gpio;

	if (value == wsc_btn_assertlvl) {
		clock_gettime(CLOCK_REALTIME, &start_ts);

		do {
			time.tv_sec = 1;
			time.tv_usec = 0;
			select(0, NULL, NULL, NULL, &time);

			bcmgpio_in (btn_mask, &value);
			value >>= wsc_btn_gpio;

			clock_gettime(CLOCK_REALTIME, &end_ts);
			if ((end_ts.tv_sec - start_ts.tv_sec) >= WSC_LONG_PRESSTIME) {
				long_btnpress = TRUE;
				break;
			}
		} while (value == wsc_btn_assertlvl);

		if (first_time)
			return WSC_NO_BTNPRESS;

		if (long_btnpress == FALSE)
			return WSC_SHORT_BTNPRESS;
		else
			return WSC_LONG_BTNPRESS;
	}
	else {
		first_time = FALSE;
		return WSC_NO_BTNPRESS;
	}
}

int Wsc_monitorPBC_loop()
{
	struct timeval time;
	//int32 timercount = 0;
	int event = WSC_EVTI_NULL;
	//while (timercount++ < WSC_DEFAULT_OW_TIME*2)
	{
		event = brcm_wsc_retrieve_event();
		if ((event != WSC_EVTI_NULL)) {
			TUTRACE((TUTRACE_INFO,"button pressed ! returning event %d\n", event));
			//timercount = 0;
			return(event);
		}

		time.tv_sec = WSC_BUTTON_POLL_TIME / 1000;
		time.tv_usec = (WSC_BUTTON_POLL_TIME % 1000) * 1000;

		/* enable timer handling */
		bcm_timer_module_enable(wsc_tmodule1, 1);

		select(0, NULL, NULL, NULL, &time);

		/* disable timer handling */
		bcm_timer_module_enable(wsc_tmodule1, 0);
	}
	return 0;
}

void set_mnConsoleFlag(int value)
{
	monitor_console = value;
}

void *monitorConsoleThread(void *p_data)
{
	char c;
	FILE *cin;
	struct termios initial_settings, new_settings;
	cin = fopen("/dev/tty", "r");
	if (!cin)
	{
		cin = fopen("/dev/console", "r");
		if (!cin)
			TUTRACE((TUTRACE_INFO,"Unable to open console\n"));
	}
	getTermSettings(fileno(cin), &initial_settings);
	new_settings = initial_settings;
	new_settings.c_lflag &= ~ICANON;
	new_settings.c_lflag &= ~ECHO;
	setTermSettings(fileno(cin), &new_settings);

	while (monitor_console)
	{
		c = getc(cin);
		
		if ((c == 'e')&&monitor_console)
		{
			TUTRACE((TUTRACE_INFO,"Get key %c from console\n", c));
			nvram_set(WSC_NV_EVENT, "1");
		}
	}

	getTermSettings(fileno(cin), &initial_settings);
	new_settings = initial_settings;
	new_settings.c_lflag |= ICANON;
	new_settings.c_lflag |= ECHO;
	setTermSettings(fileno(cin), &new_settings);
	fclose(cin);

	TUTRACE((TUTRACE_INFO,"Exit console monitor!\n", c));
	return;
}

